//<c>   Copyright 1987-1999 by Gerry J. Danen; all rights reserved.

//<t>   Display a dBase III Structure

#include"db_util.h"


/*
        Use a "PF"-like interface to select files

        Top heading: Project name
        2nd heading: File descr
*/


#define pgTITL_ "Document dBASE III Data Structures"



void    pr_rec_detail( INT16 num, dBFIELD *fld )
{
    UCHAR       *typ ;

    switch( fld->type )
    {
        case 'C' :
            typ = "Character" ;
            break ;
        case 'N' :
            typ = "Numeric" ;
            break ;
        case 'L' :
            typ = "Logical" ;
            break ;
        case 'D' :
            typ = "Date" ;
            break ;
        case 'M' :
            typ = "Memo" ;
            break ;
        default :
            typ = "? Error" ;
    }
    printf( "\n%5d  %-12s%-11s%5d", num, fld->fieldnm, typ, fld->width );
    if ( fld->type == 'N' )
        printf( "  %5d", fld->dec );
}


void    main( int argc, char *argv[] )
{
    UCHAR       filespec[FS_LEN] ;              //  File specification
    UCHAR       *dbfid ;                        //  DBF file descriptor
    UCHAR       work[STR_LEN] ;                 //  work area
    UCHAR       day_date_time[22] ;
    INT16       i, endfile, retcode ;
    INT32       ymd ;
    RECNUM      recno ;
    RECLEN      reclen ;
    BYTE        month, day, year ;
    FLDNUM      nofields ;
    dBFIELD     *fields, *fldptr ;

    tt_init( SHAREWARE_VERSION, 0, pgVERS_, pgTITL_, pgCOPR_ );
    logon(1);

    fprintf( stderr, "Redirect input from a file with DBF names, last line must be blank.\n" );
    fprintf( stderr, "Redirect output to a file or your printer.\n\n" );
    sprintf( day_date_time, "%s %s %s", TodaysDay(), TodaysDate(), TodaysTime() );
    if ( ! (fields = (dBFIELD *) malloc( (U2BYTES) sizeof(dBFIELD) * dBC_MAX_FIELDS )))
    {
        gd_exit( 1, "Not enough memory available" );
    }
    do
    {
        fprintf( stderr, "\n\nEnter name of dBASE III data file > " );
        GetLine( filespec, FS_LEN, YES, &endfile );
        st_trim( filespec );
        _strupr( filespec );
        if ( strlen(filespec) )
        {
            if ( (StrFindChar( '.', filespec )) == ERROR )      //  no dot
                st_cat( filespec, ".DBF", FS_LEN );
            retcode = dBopen( filespec, &dbfid );
            if ( retcode == SUCCESS )
            {
                retcode = dBsize( dbfid, &recno );
                if ( retcode != SUCCESS )
                {
                    fprintf( stderr, "Error %d while getting # of recs of %s\a", retcode, filespec );
                    recno = (RECNUM) 0 ;
                }
                retcode = dBgetf( dbfid, &reclen, &month, &day, &year, &nofields, fields );
                if ( retcode == SUCCESS )
                {
                    printf( "\n\t\t\t\t\t\t\t%s", day_date_time );
                    ymd = ((INT32) year * 10000L) + ((INT32) month * 100L) + (INT32) day ;
                    FormatDate( ymd, work, 1 );
                    printf( "\nStructure for database: %s",  filespec );
                    printf( "\nDate of last update   : %s",  work );
                    printf( "\nRecord length         : %9d ( + 1 byte for DELETED() )", reclen );
                    printf( "\nNumber of fields      : %9d", nofields );
                    printf( "\nNumber of records     : %9u\n", recno );
                    printf( "\nField  Name        Type       Width    Dec" );
                    fldptr = fields ;
                    for ( i=1 ; (FLDNUM) i <= nofields ; i++, fldptr++ )
                    {
                        pr_rec_detail( i, fldptr );
                    }
                    printf( "\n\f" );
                }
                else
                {
                    fprintf( stderr, "Error %d while reading header of %s\a", retcode, filespec );
                }
                dBclose( dbfid );
                dbfid = (char *) 0 ;
            }
            else
            {
                switch( retcode )
                {
                    case d_ERROR :
                        fprintf( stderr, "Could not open file %s\a", filespec );
                        break ;
                    case d_NODBF :
                        fprintf( stderr, "File %s is not a legal dBASE III file\a", filespec );
                        break ;
                    default :
                        fprintf( stderr, "Error %d while opening file %s\a", retcode, filespec );
                }
            }
        }
    }   while ( strlen(filespec) );
    free( (char *) fields );
    gd_exit( 0, "" );
}
