//<c>   Copyright 1987-1999 by Gerry J. Danen; all rights reserved.

//<t>   Verify dBASE file and Optional Index File

#include"db_util.h"

#define pgTITL_ "Verify dBASE Files"


//              ***** GLOBAL VARIABLES *****
UCHAR           dbfspec[ FS_LEN ], ndxspec[ FS_LEN ], ms[ STR_LEN ] ;
int             quiet_mode = FALSE ;

//              ***** END OF GLOBAL VARIABLES *****

void            do_error(int,unsigned char *);
void            do_help(void);
void            prt(char *str1,char *str2,char *str3);


void    do_error( INT16 ex, UCHAR *msg )
{
    prt( "\n", msg, "\a\n" );
    gd_exit( ex, "" );
}


void    do_help( void )
{
    fprintf( stdout, "DOS errorlevels:\n" );
    fprintf( stdout, "\t\t 0 - Everything is OK\n" );
    fprintf( stdout, "\t\t 1 - Index file is out-of-date\n" );
    fprintf( stdout, "\t\t 2 - Record count in header does not match physical file size\n" );
    fprintf( stdout, "\t\t 3 - Could not get last record; or read beyond last record\n" );
    fprintf( stdout, "\t\t90 - DBF file not found\n" );
    fprintf( stdout, "\t\t91 - Error opening DBF file\n" );
    fprintf( stdout, "\t\t92 - Error calculating # of records\n" );
    fprintf( stdout, "\t\t93 - Error while reading header data\n" );
    fprintf( stdout, "\t\t98 - Not enough memory\n" );
    fprintf( stdout, "\t\t99 - Syntax error\n" );
    do_error( 99, "Usage:\t\tdVERIFY dbf_file [ndx_file]\n\n\twhere\tdbf_file is a legal dBASE III file, and\n\t\tndx_file is an optional index file" );
}


void    prt( char *str1, char *str2, char *str3 )
{
    if ( ! quiet_mode )
    {
        fprintf( stdout, "%s", str1 );
        if ( strlen( str2 ) )
            fprintf( stdout, "%s", str2 );
        if ( strlen( str3 ) )
            fprintf( stdout, "%s", str3 );
    }
}


void    main( int argc, char *argv[] )
{
    UCHAR       drv[MAX_DRIVE], path[FS_LEN], fname[MAX_FNAME], fext[MAX_EXT] ;
    UCHAR       *more_less ;
    UCHAR       *record ;                       /*  record buffer       */
    UCHAR       *dbfid ;                        /*  DBF file descriptor */
    UCHAR       status  ;                       /*  record status       */
    INT16       x, retcode, do_index=FALSE ;
    RECNUM      recno, recno2 ;
    RECLEN      reclen ;
    BYTE        month, day, year ;
    FLDNUM      nofields ;
    dBFIELD     *fields ;
    FILE        *nx ;

    tt_init( SHAREWARE_VERSION, 0, pgVERS_, pgTITL_, pgCOPR_ );

    if ( argc > 1 )
    {
        if ( st_eq( argv[1], "/Q" ) )
        {
            quiet_mode = TRUE ;
            x = 2 ;
        }
        else
        {
            x = 1 ;
        }
    }
    logon(1);
    if ( argc < 2 )
        do_help();

    if ( ! (fields = (dBFIELD *) malloc( (U2BYTES) sizeof(dBFIELD) * dBC_MAX_FIELDS )))
        do_error( 98, "Sorry, not enough memory available" );

    st_cpy( dbfspec, argv[x], FS_LEN );
    if ( argc > (x+1) )
    {
        st_cpy( ndxspec, argv[x+1], FS_LEN );
        do_index = TRUE ;
    }
    else
        st_cpy( ndxspec, "", FS_LEN );
    strupr( dbfspec );
    strupr( ndxspec );
    _splitpath( dbfspec, drv, path, fname, fext );
    st_trim( fext );
    if ( ! strlen(fext) )
        st_cat( dbfspec, ".DBF", FS_LEN );
    _splitpath( ndxspec, drv, path, fname, fext );
    st_trim( fext );
    if ( ! strlen(fext) )
        st_cat( ndxspec, ".NDX", FS_LEN );

    prt( "Verifying dBASE file: ", dbfspec, "" );
    retcode = dBopen( dbfspec, &dbfid );
    if ( retcode == SUCCESS )
    {
        retcode = dBsize( dbfid, &recno );
        if ( retcode != SUCCESS )
        {
            sprintf( ms, "Error %d while getting # of recs of %s", retcode, dbfspec );
            do_error( 92, ms );
        }
        sprintf( ms, " (%u records)", recno );
        prt( ms, "", "" );
        retcode = db_Size( dbfid, &recno2 );
        if ( retcode != SUCCESS )
        {
            sprintf( ms, "Error %d while calculating # of recs of %s", retcode, dbfspec );
            do_error( 92, ms );
        }
        if ( recno != recno2 )
        {
            if ( recno < recno2 )
                more_less = "less" ;
            else
                more_less = "more" ;
            sprintf( ms, "Header indicates %s records than physical file size (%u)", more_less, recno2 );
            do_error( 2, ms );
        }
        retcode = dBgetf( dbfid, &reclen, &month, &day, &year, &nofields, fields );
        if ( retcode == SUCCESS )
        {
            if ( ! (record = (UCHAR *) malloc( reclen + 5 )) )
                do_error( 98, "Not enough memory for record" );
            if ( recno != (RECNUM) 0 )
            {
                retcode = dBgetr( dbfid, recno, record, &status );
                if ( retcode != SUCCESS )
                    do_error( 3, "Could not read last record.  Database Corrupted!" );
            }
            recno++ ;
            retcode = dBgetr( dbfid, recno, record, &status );
            if ( retcode == SUCCESS )
                if ( status != (UCHAR) 26 )
                    do_error( 3, "Could read beyond last record.  Database Corrupted!" );
        }
        else
        {
            sprintf( ms, "Error %d while reading header of %s", retcode, dbfspec );
            do_error( 93, ms );
        }
        dBclose( dbfid );
        prt( " [ok]\n", "", "" );
    }
    else
    {
        switch( retcode )
        {
            case d_ERROR :
                sprintf( ms, "Could not open file %s", dbfspec );
                break ;
            case d_NODBF :
                sprintf( ms, "File %s is not a legal dBASE III file", dbfspec );
                break ;
            default :
                sprintf( ms, "Error %d while opening file %s", retcode, dbfspec );
        }
        do_error( 91, ms );
    }
    if ( do_index )                             /*  got 2 file specs    */
    {
        prt( "Verifying Index File: ", ndxspec, "" );
        if ( (nx = fopen( ndxspec, "r" )) == NULL )
            do_error( 1, "Index file not found - indexing required" );
        retcode = recreate_ndx( dbfspec, ndxspec );
        switch ( retcode )
        {
            case -1 :
                do_error( 90, "ERROR: dbf not found" );
            case TRUE :
                do_error(  1, "Re-indexing may be required" );
            default :
                prt( " [ok]\n", "", "" );
        }
    }
    gd_exit( 0, "" );
}
